/*
** Class:  LotROLinux.PatchWindow
** Author: Alan Jackson
** Date:   2nd January 2008
**
** Control patching process for the game Lord of the Rings Online
**
** Based on a script by SNy <SNy@bmx-chemnitz.de>
**
** (C) 2007-2008 AJackson <ajackson@bcs.org.uk>
**
** This file is part of LotROLinux.
**
** LotROLinux is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** LotROLinux is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with LotROLinux.  If not, see <http://www.gnu.org/licenses/>.
*/
using Gtk;
using Glade;
using System;
using System.IO;
using System.Diagnostics;
using System.Threading;
using System.Text;
using System.Text.RegularExpressions;

namespace LotROLinux
{
	public class PatchWindow
	{
		[Widget]
        private Window winPatch;
        
		[Widget]
        private TextView txtStatus;
        
		[Widget]
        private Button btnAbort;
        
		[Widget]
        private Button btnOk;
        
		[Widget]
        private Button btnClose;

		private const string keyPatchLogErrMess = "Error accessing patch log file, please delete\n{0}{1}\nand try again.";
		private const string keyPatchErrMess = "Error launching patch process, incorrect version of patchclient.dll.";
		private const string keyPatchErr = "UNABLE TO FIND THE ENTRY POINT";
		private const string keyAppWine = "Wine";
		private const string keyAppCXGames = "CXGames";
		private const string keyAppCXOffice = "CXOffice";
		private const string keyNumber = "Number";
		private const string keyFileName = "FileName";
		private const string keyConnect = "CONNECTING";
		private const string keyCheckFiles = "CHECKING FILES";
		private const string keyCheckFiles2 = "FILES TO PATCH";
		private const string keyCheckData = "CHECKING DATA";
		private const string keyPatchFiles = "PATCHING FILES";
		private const string keyPatchData = "PATCHING DATA";
		private const string keyForwardPatch = "APPLYING";
		private const string keyDownloading = "DOWNLOADING";
		private const string keyDecrypting = "DECRYPTING";
		private const string keySize = "SIZE";
		private const string keyTemp = "/temp";
		private const string keyDone = "Done";
		private const string keyAborted = "Aborted";
		private const string keyLogFile = "patch.log";
		private const string strWinePrefix = "WINEPREFIX";

		private const string argTemplate = "rundll32.exe {0},Patch {1} --language {2} --productcode {3}";
		private const string argCXTemplate = "--bottle \"{0}\" --verbose --workdir \"{1}\" -- rundll32.exe {2},Patch {3} --language {4} --productcode {5}";
	
		private string logDir;
		private string fswPath;
		private string patchParams;
		private string patchFilesParams;
		private string patchDataParams;
		private string runDir;
		private string wineProgram;
		private string wineApp;
		private string winePrefix;
		private Process patch;
		private bool hiResEnabled;
		private bool patchFiles;
		private bool selfPatch;
		private bool aborted;
		private StreamWriter logWriter;
		
		public PatchWindow(string urlPatchServer, string prodCode,
			string language, string runDir, string patchClient,
			string wineProgram, bool hiResEnabled,
			string icoFile, string homeDir, string winePrefix,
			string wineApp, DetermineOS osType)
		{
			Glade.XML gxml;
			FileInfo tempFile;

			gxml = new Glade.XML (null, "PatchWindowSimple.glade", "winPatch", null);
			gxml.Autoconnect (this);

			if (icoFile != null)
			{
				winPatch.Icon = Gdk.Pixbuf.LoadFromResource(icoFile);
			}

			this.runDir = runDir;
			this.winePrefix = winePrefix;
			this.wineApp = wineApp;

			this.hiResEnabled = hiResEnabled;

			switch (this.wineApp)
			{
				case keyAppWine:
					this.wineProgram = wineProgram;
					patchParams = String.Format(argTemplate, patchClient, urlPatchServer,
						language, prodCode);
					break;
				case keyAppCXGames:
					if (osType.MacPathCX == "")
					{
						tempFile = new FileInfo(String.Format("{0}{1}{2}", osType.GlobalDir, osType.DirectoryCXG,
							wineProgram));

						if (tempFile.Exists)
						{
							this.wineProgram = String.Format("{0}{1}{2}", osType.GlobalDir, osType.DirectoryCXG,
								wineProgram);
						} else {
							tempFile = new FileInfo(String.Format("{0}{1}{2}", homeDir, osType.DirectoryCXG, wineProgram));
							if (tempFile.Exists)
							{
								this.wineProgram = String.Format("{0}{1}{2}", homeDir, osType.DirectoryCXG, wineProgram);
							} else {
								this.wineProgram = wineProgram;
							}
						}
					} else {
						this.wineProgram = String.Format("{0}{1}", osType.MacPathCX, wineProgram);
					}

					patchParams = String.Format(argCXTemplate, this.winePrefix, runDir, patchClient, urlPatchServer,
						language, prodCode);
					break;
				case keyAppCXOffice:
					if (osType.MacPathCX == "")
					{
						tempFile = new FileInfo(String.Format("{0}{1}{2}", osType.GlobalDir, osType.DirectoryCXO,
							wineProgram));

						if (tempFile.Exists)
						{
							this.wineProgram = String.Format("{0}{1}{2}", osType.GlobalDir, osType.DirectoryCXO,
								wineProgram);
						} else {
							tempFile = new FileInfo(String.Format("{0}{1}{2}", homeDir, osType.DirectoryCXO, wineProgram));
							if (tempFile.Exists)
							{
								this.wineProgram = String.Format("{0}{1}{2}", homeDir, osType.DirectoryCXO, wineProgram);
							} else {
								this.wineProgram = wineProgram;
							}
						}
					} else {
						this.wineProgram = String.Format("{0}{1}", osType.MacPathCX, wineProgram);
					}

					patchParams = String.Format(argCXTemplate, this.winePrefix, runDir, patchClient, urlPatchServer,
						language, prodCode);
					break;
			}

			patchFilesParams = String.Format("{0} --filesonly", patchParams);
			patchDataParams = String.Format("{0} --dataonly", patchParams);

			if (this.hiResEnabled)
			{
				patchFilesParams = String.Format("{0} --highres", patchFilesParams);
				patchDataParams = String.Format("{0} --highres", patchDataParams);
			}

			HideAbortButton();

			AddLog("Click apply to start patching");

			logDir = String.Format("{0}{1}", homeDir, osType.AppDir);

			DirectoryInfo dir = new DirectoryInfo(logDir);

			if (!dir.Exists)
			{
				dir.Create();
			}
		}

		public void Show()
		{
			winPatch.Show();
		}

		protected virtual void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			try
			{
				logWriter = new StreamWriter(String.Format("{0}{1}", logDir, keyLogFile));

				DateTime now = System.DateTime.Now;
				logWriter.WriteLine(String.Format("=== Patching started at {0} ====", now.ToLocalTime()));

				if (hiResEnabled)
				{
					logWriter.WriteLine("****** Hi-Res Graphics Enabled ******");
				}

				System.Threading.Thread patchThread = new System.Threading.Thread(Patch);

				patchFiles = true;
				selfPatch = false;
				patchThread.Start();
			} catch {
				PatchLogError();
			}
		}

		private void PatchLogError()
		{
			Gtk.Application.Invoke (delegate {
				MessageDialog md = new MessageDialog (winPatch,
					DialogFlags.DestroyWithParent,
					MessageType.Error,  ButtonsType.Ok, 
					String.Format(keyPatchLogErrMess,
						logDir, keyLogFile));
		   		md.Title = "Patch Error";
				md.Run ();
				md.Destroy();	

				btnOk.Visible = false;
				btnAbort.Visible = false;
				btnClose.Visible = true;
			});
		}

		private void PatchError()
		{
			Gtk.Application.Invoke (delegate {
				MessageDialog md = new MessageDialog (winPatch,
					DialogFlags.DestroyWithParent,
					MessageType.Error,  ButtonsType.Ok, 
					keyPatchErrMess);
		   		md.Title = "Patch Error";
				md.Run ();
				md.Destroy();	

				btnOk.Visible = false;
				btnAbort.Visible = false;
				btnClose.Visible = true;
			});
		}

		private void SelfPatch()
		{
			DateTime now = System.DateTime.Now;
			logWriter.WriteLine(String.Format("=== Self-patching started at {0} ====", now.ToLocalTime()));
		}

		protected void NewFile(object sender, FileSystemEventArgs e)
		{
			if (!e.Name.ToString().Contains(".tmp"))
			{
				AddLog(String.Format("Patching: {0}", 
					e.FullPath.ToString().Replace(fswPath + "/", "").Replace(".jrnl", ".patch")));
				logWriter.WriteLine(String.Format("Patching: {0}", 
					e.FullPath.ToString().Replace(fswPath + "/", "").Replace(".jrnl", ".patch")));
			}

			Gtk.Application.Invoke (delegate {
				while (Gtk.Application.EventsPending ())
					Gtk.Application.RunIteration();
			});
		}

		protected void BinFile(object sender, FileSystemEventArgs e)
		{
			if (!e.Name.ToString().Contains(".tmp"))
			{
				AddLog(String.Format("Applying: {0}", 
					e.FullPath.ToString().Replace(fswPath + "/bindat/", "")));
				logWriter.WriteLine(String.Format("Applying: {0}", 
					e.FullPath.ToString().Replace(fswPath + "/bindat/", "")));
			}

			Gtk.Application.Invoke (delegate {
				while (Gtk.Application.EventsPending ())
					Gtk.Application.RunIteration();
			});
		}

		private void Patch()
		{
			string errLine;
			bool patchError;
			DateTime now;

			if (patchFiles)
			{
				AddLog("Updating game files...");
				fswPath = String.Format("{0}/temp", runDir);

				DirectoryInfo temp = new DirectoryInfo(String.Format("{0}/temp", runDir));

				if (!temp.Exists)
				{
					temp.Create();
				}
			} else {
				AddLog("Updating game data files...");
				fswPath = runDir;

				DirectoryInfo bindat = new DirectoryInfo(String.Format("{0}/bindat", runDir));

				if (!bindat.Exists)
				{
					bindat.Create();
				}

				FileSystemWatcher binFSW = new FileSystemWatcher(String.Format("{0}/bindat", runDir));
				binFSW.EnableRaisingEvents = true;
				binFSW.NotifyFilter = NotifyFilters.Size;
				binFSW.Deleted += new FileSystemEventHandler(BinFile);
			}

			FileSystemWatcher myFSW = new FileSystemWatcher(fswPath);

			if (patchFiles)
			{
				myFSW.IncludeSubdirectories = true;
			} else {
				myFSW.IncludeSubdirectories = false;
			}

			myFSW.EnableRaisingEvents = true;
			myFSW.NotifyFilter = NotifyFilters.Size;
			myFSW.Created += new FileSystemEventHandler(NewFile);

			aborted = false;
			patchError = false;

			patch = null;
			patch = new Process();

			if (wineApp == keyAppWine)
			{
				patch.StartInfo.WorkingDirectory = runDir;
			}

			if (winePrefix != "")
			{
				if (wineApp == "Wine")
				{
					patch.StartInfo.EnvironmentVariables.Add
						(strWinePrefix, winePrefix);
				}
			}

			patch.StartInfo.FileName = wineProgram;
			patch.StartInfo.RedirectStandardOutput = true;
			patch.StartInfo.RedirectStandardError = true;
			patch.StartInfo.UseShellExecute = false;

			if (patchFiles)
			{
				patch.StartInfo.Arguments = patchFilesParams;
			} else {
				patch.StartInfo.Arguments = patchDataParams;
			}

			patch.Start();

			ShowAbortButton();

			while (!patch.HasExited) {
				errLine = patch.StandardError.ReadLine();

				if ((errLine != null) && (errLine.Length > 0))
				{
					if (errLine.ToUpper().Contains(keyPatchErr))
					{
						if (!patch.HasExited)
						{
							patch.Kill();
						}
						aborted = true;
						patchError = true;
					}
				}

				Gtk.Application.Invoke (delegate {
					while (Gtk.Application.EventsPending ())
						Gtk.Application.RunIteration();
				});
			}

			if (aborted){
				AddLog("Patching aborted");

				HideAbortButton();

				now = System.DateTime.Now;

				if (patchError)
				{
					logWriter.WriteLine(String.Format("=== Aborted due to patching error at {0} ====", now.ToLocalTime()));
					PatchError();
				} else {
					logWriter.WriteLine(String.Format("=== Aborted by user at {0} ====", now.ToLocalTime()));
				}

				logWriter.Close();
			} else {
				if (patchFiles)
				{
					selfPatch = SelfPatchNeeded();
					if (selfPatch)
					{
						SelfPatch();
						Patch();
					} else {
						patchFiles = false;

						if (!aborted)
						{
							Patch();
							HideAbortButton();
						}
					}
				} else {
					AddLog("Patching finished");

					now = System.DateTime.Now;
					logWriter.WriteLine(String.Format("=== Patching stopped at {0} ====", now.ToLocalTime()));

					logWriter.Close();
				}
			}
		}

		private bool SelfPatchNeeded()
		{
			bool selfPatch = false;

			DirectoryInfo temp = new DirectoryInfo(String.Format("{0}{1}", runDir, keyTemp));

			if (temp.Exists)
			{
				if ((temp.GetDirectories().Length != 0) ||
					(temp.GetFiles().Length != 0))
				{
					selfPatch = true;
				}
			}

			return selfPatch;
		}

		private void ShowAbortButton()
		{
			Gtk.Application.Invoke (delegate {
				btnOk.Visible = false;
				btnAbort.Visible = true;
				btnClose.Visible = false;
			});
		}

		private void HideAbortButton()
		{
			Gtk.Application.Invoke (delegate {
				btnOk.Visible = true;
				btnAbort.Visible = false;
				btnClose.Visible = true;
			});
		}

		private void AppendTextWithoutScroll (string text) 
		{
			Gtk.Application.Invoke (delegate {
				Gtk.TextBuffer buf = txtStatus.Buffer;

				Gtk.TextIter iter;
				buf.MoveMark(buf.InsertMark, buf.EndIter);
				if(text != null) {
					if (text.Equals ("") == false) {				
						iter = buf.EndIter;
						buf.Insert (ref iter, text);
					}
				}
				iter = buf.EndIter;
				buf.Insert (ref iter, "\n");
				buf.MoveMark (buf.SelectionBound, buf.EndIter);
			});
		}
		
		private void AddLog(string message)
		{
			Gtk.Application.Invoke (delegate {
				Gtk.TextBuffer buf = txtStatus.Buffer;

				AppendTextWithoutScroll(message);

				while (Gtk.Application.EventsPending ()) 
					Gtk.Application.RunIteration ();

				txtStatus.ScrollToMark (buf.InsertMark, 0.4, true, 0.0, 1.0);
			});
		}
		
		protected virtual void OnButtonAbortClicked (object sender, System.EventArgs e)
		{
			if (!patch.HasExited) {
				patch.Kill();
				aborted = true;
			}
			else if (patchFiles)
			{
				aborted = true;
			}
		}

		protected virtual void OnButtonCloseClicked (object sender, System.EventArgs e)
		{
			winPatch.Destroy();
		}
	}
}
		
