/*
** Class:  LotROLinux.Settings
** Author: Alan Jackson
** Date:   16th January 2008
**
** Class to hold details about the launcher settings
** for the game Lord of the Rings Online
**
** Based on a script by SNy <SNy@bmx-chemnitz.de>
**
** (C) 2007-2008 AJackson <ajackson@bcs.org.uk>
**
** This file is part of LotROLinux.
**
** LotROLinux is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** LotROLinux is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with LotROLinux.  If not, see <http://www.gnu.org/licenses/>.
*/
using System;
using System.IO;
using System.Xml;

namespace LotROLinux
{
	public class Settings
	{
		private const string dummyXml = "<Settings></Settings>";
		private const string defaultWine = "wine";
		private const string defaultDebug = "fixme-all";
		private const string fileSettings = "LotROLinux.config";
		private const string keySettings = "Settings";
		private const string keyConfiguration = "Configuration";
		private const string keyApp = "Wine.Application";
		private const string keyAppWine = "Wine";
		private const string keyAppCXGames = "CXGames";
		private const string keyAppCXOffice = "CXOffice";
		private const string keyRealm = "Realm";
		private const string keyLanguage = "Language";
		private const string keyAccount = "Account";
		private const string keyGameDir = "Game.Directory";
		private const string keyWineProg = "Wine.Program";
		private const string keyWineParams = "Wine.Parameters";
		private const string keyWineDebug = "Wine.Debug";
		private const string keyWinePrefix = "Wine.Prefix";
		private const string keyWineLog = "Wine.Log";
		private const string keyDDO = "DDO";
		private const string keyLOTRO = "LOTRO";
		private const string keyDDOTest = "DDO.Test";
		private const string keyLOTROTest = "LOTRO.Test";
		private const string keyDefaultGame = "Default.Game";
		private const string keyHiRes = "HiRes";
		private const string keyTrue = "True";
		private const string keyFalse = "False";
		private const string keyPatchClient = "PatchClient";

		private XmlDocument xSettings;
		private DetermineOS osType;
		private string baseDir;
		private string setApp;
		private string setGameDir;
		private string setLanguage;
		private string setRealm;
		private string setAccount;
		private string setWineProg;
		private string setWineParams;
		private string setWineDebug;
		private string setWinePrefix;
		private string setPatchClient;
		private string settingsFile;
		private string settingsDir;
		private string icoFile;
		private bool setWineLog;
		private bool gameDirChanged;
		private bool hiResEnabled;
		private bool existsDDO;
		private bool existsLOTRO;
		private bool usingDND;
		private bool usingTest;
		private bool focusAccount;
		private bool ignoreDefault;
		private bool abortRun;

		public Settings(string baseDir, DetermineOS osType)
		{
			this.baseDir = baseDir;
			this.osType = osType;

			usingDND = false;
			usingTest = false;
			abortRun = false;
			gameDirChanged = false;
			xSettings = null;

			settingsDir = String.Format("{0}{1}", baseDir, osType.AppDir);
			settingsFile = String.Format("{0}{1}", this.settingsDir, fileSettings);
		}

		public bool SaveSettings(bool accSettings)
		{
			try
			{		
				DirectoryInfo dir = new DirectoryInfo(settingsDir);

				if (!dir.Exists)
				{
					dir.Create();
				}
	
				if (xSettings == null)
				{
					xSettings = new XmlDocument();

					xSettings.LoadXml(dummyXml);

					XmlDeclaration xmldecl = xSettings.CreateXmlDeclaration("1.0", "utf-8", null);

					xSettings.InsertBefore(xmldecl, xSettings.DocumentElement);
				}

				XmlNodeList tempNodes;

				tempNodes = xSettings.GetElementsByTagName(keyDefaultGame);
				if (tempNodes.Count == 0)
				{
					XmlElement xDefaultGame = xSettings.CreateElement(keyDefaultGame);
					if (usingTest)
					{
						if (usingDND)
						{
							xDefaultGame.InnerText = keyDDOTest;
						} else {
							xDefaultGame.InnerText = keyLOTROTest;
						}
					} else {
						if (usingDND)
						{
							xDefaultGame.InnerText = keyDDO;
						} else {
							xDefaultGame.InnerText = keyLOTRO;
						}
					}
					xSettings.DocumentElement.AppendChild(xDefaultGame);
				} else {
					if (usingTest)
					{
						if (usingDND)
						{
							tempNodes[0].InnerText = keyDDOTest;
						} else {
							tempNodes[0].InnerText = keyLOTROTest;
						}
					} else {
						if (usingDND)
						{
							tempNodes[0].InnerText = keyDDO;
						} else {
							tempNodes[0].InnerText = keyLOTRO;
						}
					}
				}

				if (usingTest)
				{
					if (usingDND)
					{
						tempNodes = xSettings.GetElementsByTagName(keyDDOTest);
					} else {
						tempNodes = xSettings.GetElementsByTagName(keyLOTROTest);
					}
				} else {
					if (usingDND)
					{
						tempNodes = xSettings.GetElementsByTagName(keyDDO);
					} else {
						tempNodes = xSettings.GetElementsByTagName(keyLOTRO);
					}
				}

				if (tempNodes.Count != 0)
				{
					xSettings.DocumentElement.RemoveChild(tempNodes[0]);
				}

				XmlElement xGame;
				if (usingTest)
				{
					if (usingDND)
					{
						xGame = xSettings.CreateElement(keyDDOTest);
					} else {
						xGame = xSettings.CreateElement(keyLOTROTest);
					}
				} else {
					if (usingDND)
					{
						xGame = xSettings.CreateElement(keyDDO);
					} else {
						xGame = xSettings.CreateElement(keyLOTRO);
					}
				}

				XmlElement xApp = xSettings.CreateElement(keyApp);
				xApp.InnerText = setApp;
				xGame.AppendChild(xApp);

				XmlElement xWineProg = xSettings.CreateElement(keyWineProg);
				xWineProg.InnerText = setWineProg;
				xGame.AppendChild(xWineProg);

				XmlElement xWineDebug = xSettings.CreateElement(keyWineDebug);
				xWineDebug.InnerText = setWineDebug;
				xGame.AppendChild(xWineDebug);

				if (setWineParams != "")
				{
					XmlElement xWineParams = xSettings.CreateElement(keyWineParams);
					xWineParams.InnerText = setWineParams;
					xGame.AppendChild(xWineParams);
				}

				if (setWinePrefix != "")
				{
					XmlElement xWinePrefix = xSettings.CreateElement(keyWinePrefix);
					xWinePrefix.InnerText = setWinePrefix;
					xGame.AppendChild(xWinePrefix);
				}

				XmlElement xWineLog = xSettings.CreateElement(keyWineLog);
				if (setWineLog)
				{
					xWineLog.InnerText = keyTrue;
				} else {
					xWineLog.InnerText = keyFalse;
				}
				xGame.AppendChild(xWineLog);

				XmlElement xHiRes = xSettings.CreateElement(keyHiRes);
				if (hiResEnabled)
				{
					xHiRes.InnerText = keyTrue;
				} else {
					xHiRes.InnerText = keyFalse;
				}
				xGame.AppendChild(xHiRes);

				XmlElement xGameDir = xSettings.CreateElement(keyGameDir);
				xGameDir.InnerText = setGameDir;
				xGame.AppendChild(xGameDir);

				if (accSettings)
				{
					if (setRealm != "")
					{
						XmlElement xRealm = xSettings.CreateElement(keyRealm);
						xRealm.InnerText = setRealm;
						xGame.AppendChild(xRealm);
					}

					if (setLanguage != "")
					{
						XmlElement xLanguage = xSettings.CreateElement(keyLanguage);
						xLanguage.InnerText = setLanguage;
						xGame.AppendChild(xLanguage);
					}

					if (setAccount != "")
					{
						XmlElement xAccount = xSettings.CreateElement(keyAccount);
						xAccount.InnerText = setAccount;
						xGame.AppendChild(xAccount);
					}
				}

				if (usingDND)
				{
					XmlElement xPatchClient = xSettings.CreateElement(keyPatchClient);
					xPatchClient.InnerText = setPatchClient;
					xGame.AppendChild(xPatchClient);
				}

				xSettings.DocumentElement.AppendChild(xGame);

				xSettings.Save(settingsFile);

				return true;
		    }
		    catch
		    {
		    	return false;
		    }
		}

		public void ModifySettings()
		{
			gameDirChanged = false;

			SettingsWindow winSettings = new SettingsWindow(setWineProg, setWineParams, setWineDebug,
				setWinePrefix, setPatchClient, setWineLog, setGameDir, hiResEnabled, 
				usingDND, icoFile, setApp, baseDir, osType);
			winSettings.Run();
			
			setApp = winSettings.GetApp();
			setWineLog = winSettings.GetWineLog();
			setWineProg = winSettings.GetWineProgram();
			setWineParams = winSettings.GetWineParams();
			setWineDebug = winSettings.GetWineDebug();
			setWinePrefix = winSettings.GetWinePrefix();

			if (usingDND)
			{
				setPatchClient = winSettings.GetPatchClient();
			}

			if (setGameDir != winSettings.GetGameDir())
			{
				gameDirChanged = true;

				setGameDir = winSettings.GetGameDir();
			}

			hiResEnabled = winSettings.GetHiRes();
			
			winSettings.Destroy();
		}

		private bool ConvertSettings()
		{
			try
			{
				XmlDocument xTempDoc = new XmlDocument();

				xTempDoc.LoadXml(dummyXml);
				XmlDeclaration xmldecl = xTempDoc.CreateXmlDeclaration("1.0", "utf-8", null);
				xTempDoc.InsertBefore(xmldecl, xTempDoc.DocumentElement);

				XmlNodeList xDefault = xSettings.GetElementsByTagName(keyDefaultGame);
				XmlElement xEDefault = xTempDoc.CreateElement(xDefault[0].Name);
				xEDefault.InnerText = xDefault[0].InnerText;

				XmlNodeList xLOTRO = xSettings.GetElementsByTagName(keyLOTRO);
				XmlNodeList xLOTROTest = xSettings.GetElementsByTagName(keyLOTROTest);
				XmlNodeList xDDO = xSettings.GetElementsByTagName(keyDDO);
				XmlNodeList xDDOTest = xSettings.GetElementsByTagName(keyDDOTest);

				xTempDoc.DocumentElement.AppendChild(xEDefault);

				if (xLOTRO.Count != 0)
				{
					XmlNodeList xApp = xSettings.GetElementsByTagName(keyApp);
					XmlElement xEApp = xTempDoc.CreateElement(xApp[0].Name);
					xEApp.InnerText = xApp[0].InnerText;

					XmlNodeList xWineProg = xSettings.GetElementsByTagName(keyWineProg);
					XmlElement xEWineProg = xTempDoc.CreateElement(xWineProg[0].Name);
					xEWineProg.InnerText = xWineProg[0].InnerText;

					XmlNodeList xWineParams = xSettings.GetElementsByTagName(keyWineParams);
					XmlElement xEWineParams = null;
					if (xWineParams.Count != 0)
					{
						xEWineParams = xTempDoc.CreateElement(xWineParams[0].Name);
						xEWineParams.InnerText = xWineParams[0].InnerText;
					}

					XmlNodeList xWineDebug = xSettings.GetElementsByTagName(keyWineDebug);
					XmlElement xEWineDebug = xTempDoc.CreateElement(xWineDebug[0].Name);
					xEWineDebug.InnerText = xWineDebug[0].InnerText;

					XmlNodeList xWinePrefix = xSettings.GetElementsByTagName(keyWinePrefix);
					XmlElement xEWinePrefix = null;
					if (xWinePrefix.Count != 0)
					{
						xEWinePrefix = xTempDoc.CreateElement(xWinePrefix[0].Name);
						xEWinePrefix.InnerText = xWinePrefix[0].InnerText;
					}

					XmlNodeList xWineLog = xSettings.GetElementsByTagName(keyWineLog);
					XmlElement xEWineLog = xTempDoc.CreateElement(xWineLog[0].Name);
					xEWineLog.InnerText = xWineLog[0].InnerText;

					XmlElement xGame1 = xTempDoc.CreateElement(keyLOTRO);

					xGame1.AppendChild(xEApp);
					xGame1.AppendChild(xEWineProg);

					if (xWineParams.Count != 0)
					{
						xGame1.AppendChild(xEWineParams);
					}

					xGame1.AppendChild(xEWineDebug);

					if (xWinePrefix.Count != 0)
					{
						xGame1.AppendChild(xEWinePrefix);
					}

					xGame1.AppendChild(xEWineLog);

					foreach (XmlNode xNode in xLOTRO[0].ChildNodes)
					{
						XmlElement xTemp = xTempDoc.CreateElement(xNode.Name);
						xTemp.InnerText = xNode.InnerText;
						xGame1.AppendChild(xTemp);
					}

					xTempDoc.DocumentElement.AppendChild(xGame1);
				}

				if (xLOTROTest.Count != 0)
				{
					XmlNodeList xApp = xSettings.GetElementsByTagName(keyApp);
					XmlElement xEApp = xTempDoc.CreateElement(xApp[0].Name);
					xEApp.InnerText = xApp[0].InnerText;

					XmlNodeList xWineProg = xSettings.GetElementsByTagName(keyWineProg);
					XmlElement xEWineProg = xTempDoc.CreateElement(xWineProg[0].Name);
					xEWineProg.InnerText = xWineProg[0].InnerText;

					XmlNodeList xWineParams = xSettings.GetElementsByTagName(keyWineParams);
					XmlElement xEWineParams = null;
					if (xWineParams.Count != 0)
					{
						xEWineParams = xTempDoc.CreateElement(xWineParams[0].Name);
						xEWineParams.InnerText = xWineParams[0].InnerText;
					}

					XmlNodeList xWineDebug = xSettings.GetElementsByTagName(keyWineDebug);
					XmlElement xEWineDebug = xTempDoc.CreateElement(xWineDebug[0].Name);
					xEWineDebug.InnerText = xWineDebug[0].InnerText;

					XmlNodeList xWinePrefix = xSettings.GetElementsByTagName(keyWinePrefix);
					XmlElement xEWinePrefix = null;
					if (xWinePrefix.Count != 0)
					{
						xEWinePrefix = xTempDoc.CreateElement(xWinePrefix[0].Name);
						xEWinePrefix.InnerText = xWinePrefix[0].InnerText;
					}

					XmlNodeList xWineLog = xSettings.GetElementsByTagName(keyWineLog);
					XmlElement xEWineLog = xTempDoc.CreateElement(xWineLog[0].Name);
					xEWineLog.InnerText = xWineLog[0].InnerText;

					XmlElement xGame2 = xTempDoc.CreateElement(keyLOTROTest);

					xGame2.AppendChild(xEApp);
					xGame2.AppendChild(xEWineProg);

					if (xWineParams.Count != 0)
					{
						xGame2.AppendChild(xEWineParams);
					}

					xGame2.AppendChild(xEWineDebug);

					if (xWinePrefix.Count != 0)
					{
						xGame2.AppendChild(xEWinePrefix);
					}

					xGame2.AppendChild(xEWineLog);

					foreach (XmlNode xNode in xLOTROTest[0].ChildNodes)
					{
						XmlElement xTemp = xTempDoc.CreateElement(xNode.Name);
						xTemp.InnerText = xNode.InnerText;
						xGame2.AppendChild(xTemp);
					}

					xTempDoc.DocumentElement.AppendChild(xGame2);
				}

				if (xDDO.Count != 0)
				{
					XmlNodeList xApp = xSettings.GetElementsByTagName(keyApp);
					XmlElement xEApp = xTempDoc.CreateElement(xApp[0].Name);
					xEApp.InnerText = xApp[0].InnerText;

					XmlNodeList xWineProg = xSettings.GetElementsByTagName(keyWineProg);
					XmlElement xEWineProg = xTempDoc.CreateElement(xWineProg[0].Name);
					xEWineProg.InnerText = xWineProg[0].InnerText;

					XmlNodeList xWineParams = xSettings.GetElementsByTagName(keyWineParams);
					XmlElement xEWineParams = null;
					if (xWineParams.Count != 0)
					{
						xEWineParams = xTempDoc.CreateElement(xWineParams[0].Name);
						xEWineParams.InnerText = xWineParams[0].InnerText;
					}

					XmlNodeList xWineDebug = xSettings.GetElementsByTagName(keyWineDebug);
					XmlElement xEWineDebug = xTempDoc.CreateElement(xWineDebug[0].Name);
					xEWineDebug.InnerText = xWineDebug[0].InnerText;

					XmlNodeList xWinePrefix = xSettings.GetElementsByTagName(keyWinePrefix);
					XmlElement xEWinePrefix = null;
					if (xWinePrefix.Count != 0)
					{
						xEWinePrefix = xTempDoc.CreateElement(xWinePrefix[0].Name);
						xEWinePrefix.InnerText = xWinePrefix[0].InnerText;
					}

					XmlNodeList xWineLog = xSettings.GetElementsByTagName(keyWineLog);
					XmlElement xEWineLog = xTempDoc.CreateElement(xWineLog[0].Name);
					xEWineLog.InnerText = xWineLog[0].InnerText;

					XmlElement xGame3 = xTempDoc.CreateElement(keyDDO);

					xGame3.AppendChild(xEApp);
					xGame3.AppendChild(xEWineProg);

					if (xWineParams.Count != 0)
					{
						xGame3.AppendChild(xEWineParams);
					}

					xGame3.AppendChild(xEWineDebug);

					if (xWinePrefix.Count != 0)
					{
						xGame3.AppendChild(xEWinePrefix);
					}

					xGame3.AppendChild(xEWineLog);

					foreach (XmlNode xNode in xDDO[0].ChildNodes)
					{
						XmlElement xTemp = xTempDoc.CreateElement(xNode.Name);
						xTemp.InnerText = xNode.InnerText;
						xGame3.AppendChild(xTemp);
					}

					xTempDoc.DocumentElement.AppendChild(xGame3);
				}

				if (xDDOTest.Count != 0)
				{
					XmlNodeList xApp = xSettings.GetElementsByTagName(keyApp);
					XmlElement xEApp = xTempDoc.CreateElement(xApp[0].Name);
					xEApp.InnerText = xApp[0].InnerText;

					XmlNodeList xWineProg = xSettings.GetElementsByTagName(keyWineProg);
					XmlElement xEWineProg = xTempDoc.CreateElement(xWineProg[0].Name);
					xEWineProg.InnerText = xWineProg[0].InnerText;

					XmlNodeList xWineParams = xSettings.GetElementsByTagName(keyWineParams);
					XmlElement xEWineParams = null;
					if (xWineParams.Count != 0)
					{
						xEWineParams = xTempDoc.CreateElement(xWineParams[0].Name);
						xEWineParams.InnerText = xWineParams[0].InnerText;
					}

					XmlNodeList xWineDebug = xSettings.GetElementsByTagName(keyWineDebug);
					XmlElement xEWineDebug = xTempDoc.CreateElement(xWineDebug[0].Name);
					xEWineDebug.InnerText = xWineDebug[0].InnerText;

					XmlNodeList xWinePrefix = xSettings.GetElementsByTagName(keyWinePrefix);
					XmlElement xEWinePrefix = null;
					if (xWinePrefix.Count != 0)
					{
						xEWinePrefix = xTempDoc.CreateElement(xWinePrefix[0].Name);
						xEWinePrefix.InnerText = xWinePrefix[0].InnerText;
					}

					XmlNodeList xWineLog = xSettings.GetElementsByTagName(keyWineLog);
					XmlElement xEWineLog = xTempDoc.CreateElement(xWineLog[0].Name);
					xEWineLog.InnerText = xWineLog[0].InnerText;

					XmlElement xGame4 = xTempDoc.CreateElement(keyDDOTest);

					xGame4.AppendChild(xEApp);
					xGame4.AppendChild(xEWineProg);

					if (xWineParams.Count != 0)
					{
						xGame4.AppendChild(xEWineParams);
					}

					xGame4.AppendChild(xEWineDebug);

					if (xWinePrefix.Count != 0)
					{
						xGame4.AppendChild(xEWinePrefix);
					}

					xGame4.AppendChild(xEWineLog);

					foreach (XmlNode xNode in xDDOTest[0].ChildNodes)
					{
						XmlElement xTemp = xTempDoc.CreateElement(xNode.Name);
						xTemp.InnerText = xNode.InnerText;
						xGame4.AppendChild(xTemp);
					}

					xTempDoc.DocumentElement.AppendChild(xGame4);
				}

				xTempDoc.Save(settingsFile);

				return true;
			}
			catch
			{
				return false;
			}
		}

		public bool LoadSettings()
		{
			existsDDO = false;
			existsLOTRO = false;
			hiResEnabled = true;
			setApp = keyAppWine;
			setRealm = "";
			setLanguage = "";
			setAccount = "";
			setWineLog = false;
			setWineProg = defaultWine;
			setWineParams = "";
			setWineDebug = defaultDebug;
			setWinePrefix = System.Environment.GetEnvironmentVariable("WINEPREFIX");
			setPatchClient = "patchclient.dll";

			focusAccount = true;

			FileInfo configFile = new FileInfo(settingsFile);
			
			if (configFile.Exists)
			{
				try
				{
					if (xSettings == null)
					{
						xSettings = new XmlDocument();
			
						xSettings.Load(settingsFile);
					}

					XmlNodeList appSettings = xSettings.GetElementsByTagName(keyConfiguration);

					if (appSettings.Count != 0)
					{
						if (ConvertSettings())
						{
							xSettings.Load(settingsFile);
						} else {
							return false;
						}
					}
				
					appSettings = xSettings.GetElementsByTagName(keySettings)[0].ChildNodes;
			
					foreach (XmlNode appSetting in appSettings)
					{
						switch (appSetting.Name.ToString())
						{
							case keyDefaultGame:
								if (!ignoreDefault)
								{
									switch (appSetting.InnerText)
									{
										case keyDDO:
											usingDND = true;
											usingTest = false;
											break;
										case keyDDOTest:
											usingDND = true;
											usingTest = true;
											break;
										case keyLOTRO:
											usingDND = false;
											usingTest = false;
											break;
										case keyLOTROTest:
											usingDND = false;
											usingTest = true;
											break;
									}
								}
								break;
						}
					}
					
					XmlNodeList gameSettings;

					string tempGame;

					if (usingTest)
					{
						if (usingDND)
						{
							tempGame = keyDDOTest;
						} else {
							tempGame = keyLOTROTest;
						}
					} else {
						if (usingDND)
						{
							tempGame = keyDDO;
						} else {
							tempGame = keyLOTRO;
						}
					}

					gameSettings = xSettings.GetElementsByTagName(tempGame);
					if (gameSettings.Count != 0)
					{
						LoadGameSettings(gameSettings, tempGame);
					}

					if ((usingDND && !existsDDO) || (!usingDND && !existsLOTRO))
					{
						if (usingDND)
						{
							existsDDO = true;
						} else {
							existsLOTRO = true;
						}

						ModifySettings();

						return SaveSettings(false);
					}
				}
				catch
				{
					return false;
				}
			} else {
				if ((setWinePrefix != null) && (setWinePrefix != ""))
				{
					setGameDir = setWinePrefix;
				} else {
					setWinePrefix = "";
					setGameDir = baseDir;
				}

				FirstRunWindow newWizard = new FirstRunWindow();

				Gtk.ResponseType resp = (Gtk.ResponseType)newWizard.Run();

				usingDND = newWizard.GetUsingDND();
				usingTest = newWizard.GetUsingTest();

				newWizard.Destroy();

				if (resp == Gtk.ResponseType.Ok)
				{
					if (usingDND)
					{
						existsDDO = true;
					} else {
						existsLOTRO = true;
					}

					ModifySettings();

					return SaveSettings(false);
				} else {
					abortRun = true;
				}
			}

			return true;
		}

		private void LoadGameSettings(XmlNodeList gameSettings, string keyGame)
		{
			if ((keyGame.Equals(keyDDO)) || (keyGame.Equals(keyDDOTest)))
			{
				existsDDO = true;
			} else {
				existsLOTRO = true;
			}

			foreach (XmlNode appSetting in gameSettings[0].ChildNodes)
			{
				switch (appSetting.Name.ToString())
				{
					case keyWineLog:
						if (appSetting.InnerText == keyTrue)
	                    {
							setWineLog = true;
						} else {
							setWineLog = false;
						}
						break;
					case keyApp:
						setApp = appSetting.InnerText;
						break;
					case keyWineProg:
						setWineProg = appSetting.InnerText;
						break;
					case keyWineParams:
						setWineParams = appSetting.InnerText;
						break;
					case keyWineDebug:
						setWineDebug = appSetting.InnerText;
						break;
					case keyWinePrefix:
						setWinePrefix = appSetting.InnerText;
						break;
					case keyGameDir:
						setGameDir = appSetting.InnerText;
						break;
					case keyAccount:
						setAccount = appSetting.InnerText;
						focusAccount = false;
						break;
					case keyLanguage:
						setLanguage = appSetting.InnerText;
						break;
					case keyRealm:
						setRealm = appSetting.InnerText;
						break;
					case keyPatchClient:
						setPatchClient = appSetting.InnerText;
						break;
					case keyHiRes:
						if (appSetting.InnerText == keyTrue)
                        {
							hiResEnabled = true;
						} else {
							hiResEnabled = false;
						}
						break;
				}
			}

			if (setGameDir == "")
			{
				if ((setWinePrefix != null) && (setWinePrefix != ""))
				{
					setGameDir = setWinePrefix;
				} else {
					setWinePrefix = "";
					setGameDir = baseDir;
				}
			}
		}

		public string GetGameDir()
		{
			return setGameDir;
		}

		public string GetLanguage()
		{
			return setLanguage;
		}

		public string GetRealm()
		{
			return setRealm;
		}

		public string GetPatchClient()
		{
			return setPatchClient;
		}

		public bool HiResEnabled()
		{
			return hiResEnabled;
		}

		public bool UsingDND()
		{
			return usingDND;
		}

		public bool UsingTest()
		{
			return usingTest;
		}

		public bool AbortRun()
		{
			return abortRun;
		}

		public bool FocusAccount()
		{
			return focusAccount;
		}

		public bool GameDirChanged()
		{
			return gameDirChanged;
		}

		public bool GetWineLog()
		{
			return setWineLog;
		}

		public string GetWineProg()
		{
			return setWineProg;
		}

		public string GetWineParams()
		{
			return setWineParams;
		}

		public string GetWineDebug()
		{
			return setWineDebug;
		}

		public string GetWinePrefix()
		{
			return setWinePrefix;
		}

		public string GetApp()
		{
			return setApp;
		}

		public string GetAccount()
		{
			return setAccount;
		}

		public void SetAccount(string account)
		{
			setAccount = account;
		}

		public void SetRealm(string realm)
		{
			setRealm = realm;
		}

		public void SetLanguage(string language)
		{
			setLanguage = language;
		}

		public void SetIcoFile(string file)
		{
			icoFile = file;
		}

		public bool SwitchGame()
		{
			bool gameChanged = false;

			SwitchGameWindow switchWizard = new SwitchGameWindow();

			Gtk.ResponseType resp = (Gtk.ResponseType)switchWizard.Run();

			if (resp == Gtk.ResponseType.Ok)
			{
				usingDND = switchWizard.GetUsingDND();
				usingTest = switchWizard.GetUsingTest();

				ignoreDefault = true;
				gameChanged = true;
			}

			switchWizard.Destroy();

			return gameChanged;
		}
	}
}
